@@ -26,7 +26,7 @@ store the filename in the database.  Doing so loses the `ACID
 
 /* ZEROBLOB CODE */
 
-/** .. class:: zeroblob(size)
+/** .. class:: zeroblob(size: int)
 
   If you want to insert a blob into a row, you previously needed to
   supply the entire blob in one go.  To read just one byte also
@@ -34,7 +34,7 @@ store the filename in the database.  Doing so loses the `ACID
   a 100MB file you would have done::
 
      largedata=open("largefile", "rb").read()
-     cur.execute("insert into foo values(?)", (buffer(largedata),))
+     cur.execute("insert into foo values(?)", (largedata,))
 
   SQLite 3.5 allowed for incremental Blob I/O so you can read and
   write blobs in small amounts.  You cannot change the size of a blob
@@ -58,7 +58,7 @@ typedef struct
 } ZeroBlobBind;
 
 static PyObject *
-ZeroBlobBind_new(PyTypeObject *type, APSW_ARGUNUSED PyObject *args, APSW_ARGUNUSED PyObject *kwargs)
+ZeroBlobBind_new(PyTypeObject *type, PyObject *Py_UNUSED(args), PyObject *Py_UNUSED(kwargs))
 {
   ZeroBlobBind *self;
   self = (ZeroBlobBind *)type->tp_alloc(type, 0);
@@ -68,24 +68,22 @@ ZeroBlobBind_new(PyTypeObject *type, APSW_ARGUNUSED PyObject *args, APSW_ARGUNUS
 }
 
 static int
-ZeroBlobBind_init(ZeroBlobBind *self, PyObject *args, PyObject *kwargs)
+ZeroBlobBind_init(ZeroBlobBind *self, PyObject *args, PyObject *kwds)
 {
-  int n;
-  if (kwargs && PyDict_Size(kwargs) != 0)
+  int size;
+
   {
-    PyErr_Format(PyExc_TypeError, "Zeroblob constructor does not take keyword arguments");
-    return -1;
+    static char *kwlist[] = {"size", NULL};
+    Zeroblob_init_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i:" Zeroblob_init_USAGE, kwlist, &size))
+      return -1;
   }
-
-  if (!PyArg_ParseTuple(args, "i", &n))
-    return -1;
-
-  if (n < 0)
+  if (size < 0)
   {
     PyErr_Format(PyExc_TypeError, "zeroblob size must be >= 0");
     return -1;
   }
-  self->blobsize = n;
+  self->blobsize = size;
 
   return 0;
 }
@@ -97,17 +95,16 @@ ZeroBlobBind_init(ZeroBlobBind *self, PyObject *args, PyObject *kwargs)
 static PyObject *
 ZeroBlobBind_len(ZeroBlobBind *self)
 {
-  return PyInt_FromLong(self->blobsize);
+  return PyLong_FromLong(self->blobsize);
 }
 
 static PyMethodDef ZeroBlobBind_methods[] = {
     {"length", (PyCFunction)ZeroBlobBind_len, METH_NOARGS,
-     "Size of zero blob"},
+     Zeroblob_length_DOC},
     {0, 0, 0, 0}};
 
 static PyTypeObject ZeroBlobBindType = {
-    APSW_PYTYPE_INIT
-    "apsw.zeroblob",                                                        /*tp_name*/
+    PyVarObject_HEAD_INIT(NULL, 0) "apsw.zeroblob",                         /*tp_name*/
     sizeof(ZeroBlobBind),                                                   /*tp_basicsize*/
     0,                                                                      /*tp_itemsize*/
     0,                                                                      /*tp_dealloc*/
@@ -126,7 +123,7 @@ static PyTypeObject ZeroBlobBindType = {
     0,                                                                      /*tp_setattro*/
     0,                                                                      /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_VERSION_TAG, /*tp_flags*/
-    "ZeroBlobBind object",                                                  /* tp_doc */
+    Zeroblob_init_DOC,                                                      /* tp_doc */
     0,                                                                      /* tp_traverse */
     0,                                                                      /* tp_clear */
     0,                                                                      /* tp_richcompare */
@@ -151,8 +148,9 @@ static PyTypeObject ZeroBlobBindType = {
     0,                                                                      /* tp_cache */
     0,                                                                      /* tp_subclasses */
     0,                                                                      /* tp_weaklist */
-    0                                                                       /* tp_del */
-    APSW_PYTYPE_VERSION};
+    0,                                                                      /* tp_del */
+    PyType_TRAILER
+};
 
 /* BLOB TYPE */
 struct APSWBlob
@@ -171,7 +169,7 @@ static PyTypeObject APSWBlobType;
 
 /* BLOB CODE */
 
-/** .. class:: blob
+/** .. class:: Blob
 
   This object is created by :meth:`Connection.blobopen` and provides
   access to a blob in the database.  It behaves like a Python file.
@@ -280,20 +278,18 @@ APSWBlob_length(APSWBlob *self)
   return PyLong_FromLong(sqlite3_blob_bytes(self->pBlob));
 }
 
-/** .. method:: read([nbytes]) -> bytes
+/** .. method:: read(length: int = -1) -> bytes
 
   Reads amount of data requested, or till end of file, whichever is
-  earlier. Attempting to read beyond the end of the blob returns the
-  empty string/bytes, in the same manner as end of file on normal file
-  objects.
-
-  :rtype: (Python 2) string  (Python 3) bytes
+  earlier. Attempting to read beyond the end of the blob returns an
+  empty bytes in the same manner as end of file on normal file
+  objects.  Negative numbers read remaining data.
 
   -* sqlite3_blob_read
 */
 
 static PyObject *
-APSWBlob_read(APSWBlob *self, PyObject *args)
+APSWBlob_read(APSWBlob *self, PyObject *args, PyObject *kwds)
 {
   int length = -1;
   int res;
@@ -308,8 +304,12 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
      bytes from /dev/zero on a 64 bit machine with lots of swap to see
      why).  In any event we remain consistent with Python file
      objects */
-  if (!PyArg_ParseTuple(args, "|i:read(numbytes=remaining)", &length))
-    return NULL;
+  {
+    static char *kwlist[] = {"length", NULL};
+    Blob_read_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|i:" Blob_read_USAGE, kwlist, &length))
+      return NULL;
+  }
 
   if (
       (self->curoffset == sqlite3_blob_bytes(self->pBlob)) /* eof */
@@ -321,6 +321,7 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
     length = sqlite3_blob_bytes(self->pBlob) - self->curoffset;
 
   /* trying to read more than is in the blob? */
+  /* ::TODO:: use 64 arithmetic to avoid overflow */
   if (self->curoffset + length > sqlite3_blob_bytes(self->pBlob))
     length = sqlite3_blob_bytes(self->pBlob) - self->curoffset;
 
@@ -346,15 +347,15 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
   return buffy;
 }
 
-/** .. method:: readinto(buffer[, offset=0, length=remaining-buffer]) -> None
+/** .. method:: readinto(buffer: Union[bytearray, array[Any], memoryview], offset: int = 0, length: int = -1) -> None
 
   Reads from the blob into a buffer you have supplied.  This method is
   useful if you already have a buffer like object that data is being
   assembled in, and avoids allocating results in :meth:`blob.read` and
   then copying into buffer.
 
-  :param buffer: A writable buffer like object.  In Python 2.6 onwards
-                 there is a bytearray type that is very useful.
+  :param buffer: A writable buffer like object.
+                 There is a bytearray type that is very useful.
                  :class:`array.array` also works.
 
   :param offset: The position to start writing into the buffer
@@ -369,86 +370,55 @@ APSWBlob_read(APSWBlob *self, PyObject *args)
 */
 
 static PyObject *
-APSWBlob_readinto(APSWBlob *self, PyObject *args)
+APSWBlob_readinto(APSWBlob *self, PyObject *args, PyObject *kwds)
 {
-  int length;
-  int res;
-  Py_ssize_t offset, lengthwanted;
-  PyObject *wbuf = NULL;
-  PyObject *errorexitval = NULL;
+  int res = SQLITE_OK;
+  long long offset = 0, length = -1;
+  PyObject *buffer = NULL;
 
   int aswb;
-  void *buffer;
-  Py_ssize_t bufsize;
 
   int bloblen;
-#if PY_VERSION_HEX >= 0x03000000
   Py_buffer py3buffer;
-#endif
 
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
+  {
+    static char *kwlist[] = {"buffer", "offset", "length", NULL};
+    Blob_readinto_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|LL:" Blob_readinto_USAGE, kwlist, &buffer, &offset, &length))
+      return NULL;
+  }
 
-  /* To get Py_ssize_t we need "n" format but that only exists in
-     Python 2.5 plus */
-
-  if (!PyArg_ParseTuple(args, "O|"
-#if PY_VERSION_HEX < 0x02050000
-                              "i"
-#else
-                              "n"
-#endif
-                              "i:readinto(wbuf, offset=1, length=wbufremaining)",
-                        &wbuf, &offset, &length))
-    return NULL;
-
-#define ERREXIT(x)      \
-  do                    \
-  {                     \
-    errorexitval = (x); \
-    goto errorexit;     \
+#define ERREXIT(x)  \
+  do                \
+  {                 \
+    x;              \
+    goto errorexit; \
   } while (0)
 
-#if PY_VERSION_HEX < 0x03000000
-  aswb = PyObject_AsWriteBuffer(wbuf, &buffer, &bufsize);
-  if (aswb)
-    return NULL;
-#else
   memset(&py3buffer, 0, sizeof(py3buffer));
-  aswb = PyObject_GetBuffer(wbuf, &py3buffer, PyBUF_WRITABLE | PyBUF_SIMPLE);
+  aswb = PyObject_GetBuffer(buffer, &py3buffer, PyBUF_WRITABLE | PyBUF_SIMPLE);
   if (aswb)
     return NULL;
-  buffer = py3buffer.buf;
-  bufsize = py3buffer.len;
-#endif
-
-  /* Although a lot of these checks could be combined into a single
-     one, they are kept separate so that we can verify they have each
-     been exercised with code coverage checks */
-
-  if (PyTuple_GET_SIZE(args) < 2)
-    offset = 0;
 
   bloblen = sqlite3_blob_bytes(self->pBlob);
 
-  if (offset < 0 || offset > bufsize)
+  if (length < 0)
+    length = py3buffer.len - offset;
+
+  if (offset < 0 || offset > py3buffer.len)
     ERREXIT(PyErr_Format(PyExc_ValueError, "offset is less than zero or beyond end of buffer"));
 
-  if (PyTuple_GET_SIZE(args) < 3)
-    lengthwanted = bufsize - offset;
-  else
-    lengthwanted = length;
-
-  if (lengthwanted < 0)
-    ERREXIT(PyErr_Format(PyExc_ValueError, "Length wanted is negative"));
-
-  if (offset + lengthwanted > bufsize)
+  if (offset + length > py3buffer.len)
     ERREXIT(PyErr_Format(PyExc_ValueError, "Data would go beyond end of buffer"));
 
-  if (lengthwanted > bloblen - self->curoffset)
+  if (length > bloblen - self->curoffset)
     ERREXIT(PyErr_Format(PyExc_ValueError, "More data requested than blob length"));
 
-  PYSQLITE_BLOB_CALL(res = sqlite3_blob_read(self->pBlob, (char *)buffer + offset, lengthwanted, self->curoffset));
+  APSW_FAULT_INJECT(BlobReadIntoPyError,
+    PYSQLITE_BLOB_CALL(res = sqlite3_blob_read(self->pBlob, (char *)(py3buffer.buf) + offset, length, self->curoffset)),
+    PyErr_NoMemory());
   if (PyErr_Occurred())
     ERREXIT(NULL);
 
@@ -457,22 +427,18 @@ APSWBlob_readinto(APSWBlob *self, PyObject *args)
     SET_EXC(res, self->connection->db);
     ERREXIT(NULL);
   }
-  self->curoffset += lengthwanted;
+  self->curoffset += length;
 
-#if PY_VERSION_HEX >= 0x03000000
   PyBuffer_Release(&py3buffer);
-#endif
   Py_RETURN_NONE;
 
 errorexit:
-#if PY_VERSION_HEX >= 0x03000000
   PyBuffer_Release(&py3buffer);
-#endif
-  return errorexitval;
+  return NULL;
 #undef ERREXIT
 }
 
-/** .. method:: seek(offset[, whence=0]) -> None
+/** .. method:: seek(offset: int, whence: int = 0) -> None
 
   Changes current position to *offset* biased by *whence*.
 
@@ -484,15 +450,18 @@ errorexit:
 */
 
 static PyObject *
-APSWBlob_seek(APSWBlob *self, PyObject *args)
+APSWBlob_seek(APSWBlob *self, PyObject *args, PyObject *kwds)
 {
   int offset, whence = 0;
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
 
-  if (!PyArg_ParseTuple(args, "i|i:seek(offset,whence=0)", &offset, &whence))
-    return NULL;
-
+  {
+    static char *kwlist[] = {"offset", "whence", NULL};
+    Blob_seek_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i|i:" Blob_seek_USAGE, kwlist, &offset, &whence))
+      return NULL;
+  }
   switch (whence)
   {
   default:
@@ -531,11 +500,11 @@ APSWBlob_tell(APSWBlob *self)
   return PyLong_FromLong(self->curoffset);
 }
 
-/** .. method:: write(data) -> None
+/** .. method:: write(data: bytes) -> None
 
   Writes the data to the blob.
 
-  :param data: (Python 2) buffer or string. (Python 3) buffer or bytes.
+  :param data: bytes to write
 
   :raises TypeError: Wrong data type
 
@@ -547,63 +516,57 @@ APSWBlob_tell(APSWBlob *self)
   -* sqlite3_blob_write
 */
 static PyObject *
-APSWBlob_write(APSWBlob *self, PyObject *obj)
+APSWBlob_write(APSWBlob *self, PyObject *args, PyObject *kwds)
 {
-  const void *buffer = 0;
-  Py_ssize_t buflen;
-  int res, asrb;
-  PyObject *errval = NULL;
-  READBUFFERVARS;
+  int ok = 0, res = SQLITE_OK;
+  Py_buffer data;
 
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
 
-  /* we support buffers and string for the object */
-  if (!PyUnicode_Check(obj) && compat_CheckReadBuffer(obj))
   {
-    compat_PyObjectReadBuffer(obj);
-
-    APSW_FAULT_INJECT(BlobWriteAsReadBufFails, , ENDREADBUFFER; (PyErr_NoMemory(), asrb = -1));
-
-    if (asrb != 0)
+    static char *kwlist[] = {"data", NULL};
+    Blob_write_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "y*:" Blob_write_USAGE, kwlist, &data))
       return NULL;
   }
-  else
-    return PyErr_Format(PyExc_TypeError, "Parameter should be bytes/string or buffer");
 
-  if (((int)(buflen + self->curoffset)) < self->curoffset)
+  if (((int)(data.len + self->curoffset)) < self->curoffset)
   {
-    errval = PyErr_Format(PyExc_ValueError, "Data is too large (integer wrap)");
-    goto errout;
+    PyErr_Format(PyExc_ValueError, "Data is too large (integer wrap)");
+    goto finally;
   }
 
-  if (((int)(buflen + self->curoffset)) > sqlite3_blob_bytes(self->pBlob))
+  if (((int)(data.len + self->curoffset)) > sqlite3_blob_bytes(self->pBlob))
   {
-    errval = PyErr_Format(PyExc_ValueError, "Data would go beyond end of blob");
-    goto errout;
+    PyErr_Format(PyExc_ValueError, "Data would go beyond end of blob");
+    goto finally;
   }
 
-  PYSQLITE_BLOB_CALL(res = sqlite3_blob_write(self->pBlob, buffer, buflen, self->curoffset));
+  APSW_FAULT_INJECT(BlobWritePyError,
+    PYSQLITE_BLOB_CALL(res = sqlite3_blob_write(self->pBlob, data.buf, data.len, self->curoffset)),
+    PyErr_NoMemory());
   if (PyErr_Occurred())
-    goto errout;
+    goto finally;
 
   if (res != SQLITE_OK)
   {
     SET_EXC(res, self->connection->db);
-    goto errout;
+    goto finally;
   }
-  else
-    self->curoffset += buflen;
+  self->curoffset += data.len;
   assert(self->curoffset <= sqlite3_blob_bytes(self->pBlob));
-  ENDREADBUFFER;
-  Py_RETURN_NONE;
+  ok = 1;
 
-errout:
-  ENDREADBUFFER;
-  return errval;
+finally:
+  PyBuffer_Release(&data);
+  if (ok)
+    Py_RETURN_NONE;
+  else
+    return NULL;
 }
 
-/** .. method:: close([force=False])
+/** .. method:: close(force: bool = False) -> None
 
   Closes the blob.  Note that even if an error occurs the blob is
   still closed.
@@ -626,16 +589,19 @@ errout:
 */
 
 static PyObject *
-APSWBlob_close(APSWBlob *self, PyObject *args)
+APSWBlob_close(APSWBlob *self, PyObject *args, PyObject *kwds)
 {
   int setexc;
   int force = 0;
 
   CHECK_USE(NULL);
 
-  if (args && !PyArg_ParseTuple(args, "|i:close(force=False)", &force))
-    return NULL;
-
+  {
+    static char *kwlist[] = {"force", NULL};
+    Blob_close_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&:" Blob_close_USAGE, kwlist, argcheck_bool, &force))
+      return NULL;
+  }
   setexc = APSWBlob_close_internal(self, !!force);
 
   if (setexc)
@@ -644,7 +610,7 @@ APSWBlob_close(APSWBlob *self, PyObject *args)
   Py_RETURN_NONE;
 }
 
-/** .. method:: __enter__() -> context
+/** .. method:: __enter__() -> Blob
 
   You can use a blob as a `context manager
   <http://docs.python.org/reference/datamodel.html#with-statement-context-managers>`_
@@ -670,7 +636,7 @@ APSWBlob_enter(APSWBlob *self)
   return (PyObject *)self;
 }
 
-/** .. method:: __exit__() -> False
+/** .. method:: __exit__() -> Literal[False]
 
   Implements context manager in conjunction with
   :meth:`~blob.__enter__`.  Any exception that happened in the
@@ -678,21 +644,20 @@ APSWBlob_enter(APSWBlob *self)
 */
 
 static PyObject *
-APSWBlob_exit(APSWBlob *self, APSW_ARGUNUSED PyObject *args)
+APSWBlob_exit(APSWBlob *self, PyObject *Py_UNUSED(args))
 {
-  PyObject *res;
+  int setexc;
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
 
-  res = APSWBlob_close(self, NULL);
-  Py_XDECREF(res);
-  if (!res)
+  setexc = APSWBlob_close_internal(self, 0);
+  if (setexc)
     return NULL;
 
   Py_RETURN_FALSE;
 }
 
-/** .. method:: reopen(rowid)
+/** .. method:: reopen(rowid: int) -> None
 
   Change this blob object to point to a different row.  It can be
   faster than closing an existing blob an opening a new one.
@@ -701,7 +666,7 @@ APSWBlob_exit(APSWBlob *self, APSW_ARGUNUSED PyObject *args)
 */
 
 static PyObject *
-APSWBlob_reopen(APSWBlob *self, PyObject *arg)
+APSWBlob_reopen(APSWBlob *self, PyObject *args, PyObject *kwds)
 {
   int res;
   long long rowid;
@@ -709,20 +674,12 @@ APSWBlob_reopen(APSWBlob *self, PyObject *arg)
   CHECK_USE(NULL);
   CHECK_BLOB_CLOSED;
 
-#if PY_MAJOR_VERSION < 3
-  if (PyInt_Check(arg))
-    rowid = PyInt_AS_LONG(arg);
-  else
-#endif
-      if (PyLong_Check(arg))
   {
-    rowid = PyLong_AsLongLong(arg);
-    if (PyErr_Occurred())
+    static char *kwlist[] = {"rowid", NULL};
+    Blob_reopen_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "L:" Blob_reopen_USAGE, kwlist, &rowid))
       return NULL;
   }
-  else
-    return PyErr_Format(PyExc_TypeError, "blob reopen argument must be a number");
-
   /* no matter what happens we always reset current offset */
   self->curoffset = 0;
 
@@ -740,31 +697,30 @@ APSWBlob_reopen(APSWBlob *self, PyObject *arg)
 
 static PyMethodDef APSWBlob_methods[] = {
     {"length", (PyCFunction)APSWBlob_length, METH_NOARGS,
-     "Returns length in bytes of the blob"},
-    {"read", (PyCFunction)APSWBlob_read, METH_VARARGS,
-     "Reads data from the blob"},
-    {"readinto", (PyCFunction)APSWBlob_readinto, METH_VARARGS,
-     "Reads data from the blob into a provided buffer"},
-    {"seek", (PyCFunction)APSWBlob_seek, METH_VARARGS,
-     "Seeks to a position in the blob"},
+     Blob_length_DOC},
+    {"read", (PyCFunction)APSWBlob_read, METH_VARARGS | METH_KEYWORDS,
+     Blob_read_DOC},
+    {"readinto", (PyCFunction)APSWBlob_readinto, METH_VARARGS | METH_KEYWORDS,
+     Blob_readinto_DOC},
+    {"seek", (PyCFunction)APSWBlob_seek, METH_VARARGS | METH_KEYWORDS,
+     Blob_seek_DOC},
     {"tell", (PyCFunction)APSWBlob_tell, METH_NOARGS,
-     "Returns current blob offset"},
-    {"write", (PyCFunction)APSWBlob_write, METH_O,
-     "Writes data to blob"},
-    {"reopen", (PyCFunction)APSWBlob_reopen, METH_O,
-     "Changes the blob to point to a different row"},
-    {"close", (PyCFunction)APSWBlob_close, METH_VARARGS,
-     "Closes blob"},
+     Blob_tell_DOC},
+    {"write", (PyCFunction)APSWBlob_write, METH_VARARGS | METH_KEYWORDS,
+     Blob_write_DOC},
+    {"reopen", (PyCFunction)APSWBlob_reopen, METH_VARARGS | METH_KEYWORDS,
+     Blob_reopen_DOC},
+    {"close", (PyCFunction)APSWBlob_close, METH_VARARGS | METH_KEYWORDS,
+     Blob_close_DOC},
     {"__enter__", (PyCFunction)APSWBlob_enter, METH_NOARGS,
-     "Context manager entry"},
+     Blob_enter_DOC},
     {"__exit__", (PyCFunction)APSWBlob_exit, METH_VARARGS,
-     "Context manager exit"},
+     Blob_exit_DOC},
     {0, 0, 0, 0} /* Sentinel */
 };
 
 static PyTypeObject APSWBlobType = {
-    APSW_PYTYPE_INIT
-    "apsw.blob",                                      /*tp_name*/
+    PyVarObject_HEAD_INIT(NULL, 0) "apsw.Blob",       /*tp_name*/
     sizeof(APSWBlob),                                 /*tp_basicsize*/
     0,                                                /*tp_itemsize*/
     (destructor)APSWBlob_dealloc,                     /*tp_dealloc*/
@@ -783,7 +739,7 @@ static PyTypeObject APSWBlobType = {
     0,                                                /*tp_setattro*/
     0,                                                /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_VERSION_TAG, /*tp_flags*/
-    "APSW blob object",                               /* tp_doc */
+    Blob_init_DOC,                                    /* tp_doc */
     0,                                                /* tp_traverse */
     0,                                                /* tp_clear */
     0,                                                /* tp_richcompare */
@@ -808,5 +764,6 @@ static PyTypeObject APSWBlobType = {
     0,                                                /* tp_cache */
     0,                                                /* tp_subclasses */
     0,                                                /* tp_weaklist */
-    0                                                 /* tp_del */
-    APSW_PYTYPE_VERSION};
+    0,                                                /* tp_del */
+    PyType_TRAILER
+};
