@@ -4,8 +4,6 @@
    See the accompanying LICENSE file.
 */
 
-#ifdef EXPERIMENTAL
-
 /**
 
 .. _virtualtables:
@@ -158,19 +156,19 @@ apswvtabCreateOrConnect(sqlite3 *db,
   schema = PySequence_GetItem(pyres, 0);
   if (!schema)
     goto pyexception;
-
+  if (!PyUnicode_Check(schema)) {
+    PyErr_Format(PyExc_TypeError, "Expected string for schema");
+    goto pyexception;
+  }
   {
-    PyObject *utf8schema = getutf8string(schema);
-    const char *cp_utf8schema;
-    if (!utf8schema)
+    const char *utf8schema = PyUnicode_AsUTF8(schema);
+    if(!utf8schema)
       goto pyexception;
-    cp_utf8schema = PyBytes_AsString(utf8schema);
-    _PYSQLITE_CALL_E(db, res = sqlite3_declare_vtab(db, cp_utf8schema));
-    Py_DECREF(utf8schema);
+    _PYSQLITE_CALL_E(db, res = sqlite3_declare_vtab(db, utf8schema));
     if (res != SQLITE_OK)
     {
       SET_EXC(res, db);
-      AddTraceBackHere(__FILE__, __LINE__, create_or_connect_strings[stringindex].declarevtabtracebackname, "{s: O}", "schema", schema);
+      AddTraceBackHere(__FILE__, __LINE__, create_or_connect_strings[stringindex].declarevtabtracebackname, "{s: O}", "schema", OBJ(schema));
       goto finally;
     }
   }
@@ -185,7 +183,7 @@ apswvtabCreateOrConnect(sqlite3 *db,
 pyexception: /* we had an exception in python code */
   res = MakeSqliteMsgFromPyException(errmsg);
   AddTraceBackHere(__FILE__, __LINE__, create_or_connect_strings[stringindex].pyexceptionname,
-                   "{s: s, s: s, s: s, s: O}", "modulename", argv[0], "database", argv[1], "tablename", argv[2], "schema", schema ? schema : Py_None);
+                   "{s: s, s: s, s: s, s: O}", "modulename", argv[0], "database", argv[1], "tablename", argv[2], "schema", OBJ(schema));
 
 finally: /* cleanup */
   Py_XDECREF(args);
@@ -211,7 +209,7 @@ finally: /* cleanup */
     find and reuse that on the subsequent :meth:`~VTModule.Connect`
     calls.
 
-    The corresponding call is :meth:`VTTable.Disconnect`.  If you have a simple virtual table implemtation, then just
+    The corresponding call is :meth:`VTTable.Disconnect`.  If you have a simple virtual table implementation, then just
     set :meth:`~VTModule.Connect` to be the same as :meth:`~VTModule.Create`::
 
       class MyModule:
@@ -359,7 +357,7 @@ apswvtabDestroyOrDisconnect(sqlite3_vtab *pVtab, int stringindex)
 
   /* pyexception:  we had an exception in python code */
   sqliteres = MakeSqliteMsgFromPyException(&(pVtab->zErrMsg));
-  AddTraceBackHere(__FILE__, __LINE__, destroy_disconnect_strings[stringindex].pyexceptionname, "{s: O}", "self", vtable);
+  AddTraceBackHere(__FILE__, __LINE__, destroy_disconnect_strings[stringindex].pyexceptionname, "{s: O}", "self", OBJ(vtable));
 
 finally:
   Py_XDECREF(res);
@@ -428,7 +426,7 @@ apswvtabDisconnect(sqlite3_vtab *pVTab)
 
   **constraints**
 
-  You will be passed the contraints as a sequence of tuples containing two
+  You will be passed the constraints as a sequence of tuples containing two
   items. The first item is the column number and the second item is
   the operation.
 
@@ -642,7 +640,7 @@ apswvtabBestIndex(sqlite3_vtab *pVtab, sqlite3_index_info *indexinfo)
   if (!PySequence_Check(res) || PySequence_Size(res) > 5)
   {
     PyErr_Format(PyExc_TypeError, "Bad result from BestIndex.  It should be a sequence of up to 5 items");
-    AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex.result_check", "{s: O, s: O}", "self", vtable, "result", res);
+    AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex.result_check", "{s: O, s: O}", "self", vtable, "result", OBJ(res));
     goto pyexception;
   }
 
@@ -657,7 +655,7 @@ apswvtabBestIndex(sqlite3_vtab *pVtab, sqlite3_index_info *indexinfo)
     {
       PyErr_Format(PyExc_TypeError, "Bad constraints (item 0 in BestIndex return).  It should be a sequence the same length as the constraints passed in (%d) items", nconstraints);
       AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex.result_indices", "{s: O, s: O, s: O}",
-                       "self", vtable, "result", res, "indices", indices);
+                       "self", vtable, "result", OBJ(res), "indices", OBJ(indices));
       goto pyexception;
     }
     /* iterate through the items - i is the SQLite sequence number and j is the apsw one (usable entries) */
@@ -678,9 +676,9 @@ apswvtabBestIndex(sqlite3_vtab *pVtab, sqlite3_index_info *indexinfo)
         continue;
       }
       /* or an integer */
-      if (PyIntLong_Check(constraint))
+      if (PyLong_Check(constraint))
       {
-        indexinfo->aConstraintUsage[i].argvIndex = PyIntLong_AsLong(constraint) + 1;
+        indexinfo->aConstraintUsage[i].argvIndex = PyLong_AsLong(constraint) + 1;
         Py_DECREF(constraint);
         continue;
       }
@@ -689,7 +687,7 @@ apswvtabBestIndex(sqlite3_vtab *pVtab, sqlite3_index_info *indexinfo)
       {
         PyErr_Format(PyExc_TypeError, "Bad constraint (#%d) - it should be one of None, an integer or a tuple of an integer and a boolean", j);
         AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex.result_constraint", "{s: O, s: O, s: O, s: O}",
-                         "self", vtable, "result", res, "indices", indices, "constraint", constraint);
+                         "self", vtable, "result", OBJ(res), "indices", OBJ(indices), "constraint", OBJ(constraint));
         Py_DECREF(constraint);
         goto pyexception;
       }
@@ -697,17 +695,17 @@ apswvtabBestIndex(sqlite3_vtab *pVtab, sqlite3_index_info *indexinfo)
       omit = PySequence_GetItem(constraint, 1);
       if (!argvindex || !omit)
         goto constraintfail;
-      if (!PyIntLong_Check(argvindex))
+      if (!PyLong_Check(argvindex))
       {
         PyErr_Format(PyExc_TypeError, "argvindex for constraint #%d should be an integer", j);
         AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex.result_constraint_argvindex", "{s: O, s: O, s: O, s: O, s: O}",
-                         "self", vtable, "result", res, "indices", indices, "constraint", constraint, "argvindex", argvindex);
+                         "self", vtable, "result", OBJ(res), "indices", OBJ(indices), "constraint", OBJ(constraint), "argvindex", OBJ(argvindex));
         goto constraintfail;
       }
       omitv = PyObject_IsTrue(omit);
       if (omitv == -1)
         goto constraintfail;
-      indexinfo->aConstraintUsage[i].argvIndex = PyIntLong_AsLong(argvindex) + 1;
+      indexinfo->aConstraintUsage[i].argvIndex = PyLong_AsLong(argvindex) + 1;
       indexinfo->aConstraintUsage[i].omit = omitv;
       Py_DECREF(constraint);
       Py_DECREF(argvindex);
@@ -731,14 +729,14 @@ apswvtabBestIndex(sqlite3_vtab *pVtab, sqlite3_index_info *indexinfo)
       goto pyexception;
     if (idxnum != Py_None)
     {
-      if (!PyIntLong_Check(idxnum))
+      if (!PyLong_Check(idxnum))
       {
         PyErr_Format(PyExc_TypeError, "idxnum must be an integer");
-        AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex.result_indexnum", "{s: O, s: O, s: O}", "self", vtable, "result", res, "indexnum", idxnum);
+        AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex.result_indexnum", "{s: O, s: O, s: O}", "self", vtable, "result", OBJ(res), "indexnum", OBJ(idxnum));
         Py_DECREF(idxnum);
         goto pyexception;
       }
-      indexinfo->idxNum = PyIntLong_AsLong(idxnum);
+      indexinfo->idxNum = PyLong_AsLong(idxnum);
     }
     Py_DECREF(idxnum);
   }
@@ -747,23 +745,21 @@ apswvtabBestIndex(sqlite3_vtab *pVtab, sqlite3_index_info *indexinfo)
   if (PySequence_Size(res) < 3)
     goto finally;
   {
-    PyObject *utf8str = NULL, *idxstr = NULL;
+    PyObject *idxstr = NULL;
     idxstr = PySequence_GetItem(res, 2);
     if (!idxstr)
       goto pyexception;
     if (idxstr != Py_None)
     {
-      utf8str = getutf8string(idxstr);
-      if (!utf8str)
-      {
+      if(!PyUnicode_Check(idxstr)) {
+        PyErr_Format(PyExc_TypeError, "Expected a string for idxStr");
         Py_DECREF(idxstr);
         goto pyexception;
       }
-      indexinfo->idxStr = sqlite3_mprintf("%s", PyBytes_AsString(utf8str));
+      indexinfo->idxStr = sqlite3_mprintf("%s", PyUnicode_AsUTF8(idxstr));
       indexinfo->needToFreeIdxStr = 1;
     }
-    Py_XDECREF(utf8str);
-    Py_DECREF(idxstr);
+
   }
 
   /* item 3 is orderByConsumed */
@@ -816,7 +812,7 @@ apswvtabBestIndex(sqlite3_vtab *pVtab, sqlite3_index_info *indexinfo)
 pyexception: /* we had an exception in python code */
   assert(PyErr_Occurred());
   sqliteres = MakeSqliteMsgFromPyException(&(pVtab->zErrMsg));
-  AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex", "{s: O, s: O, s: (OO)}", "self", vtable, "result", res ? res : Py_None, "args", constraints ? constraints : Py_None, orderbys ? orderbys : Py_None);
+  AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xBestIndex", "{s: O, s: O, s: (OO)}", "self", vtable, "result", OBJ(res), "args", OBJ(constraints), OBJ(orderbys));
 
 finally:
   Py_XDECREF(indices);
@@ -955,7 +951,7 @@ apswvtabOpen(sqlite3_vtab *pVtab, sqlite3_vtab_cursor **ppCursor)
 pyexception: /* we had an exception in python code */
   assert(PyErr_Occurred());
   sqliteres = MakeSqliteMsgFromPyException(&(pVtab->zErrMsg));
-  AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xOpen", "{s: O}", "self", vtable);
+  AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xOpen", "{s: O}", "self", OBJ(vtable));
 
 finally:
   Py_XDECREF(res);
@@ -1091,7 +1087,7 @@ apswvtabUpdate(sqlite3_vtab *pVtab, int argc, sqlite3_value **argv, sqlite3_int6
     Py_DECREF(rowid);
     if (PyErr_Occurred())
     {
-      AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xUpdateInsertRow.ReturnedValue", "{s: O}", "result", rowid);
+      AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xUpdateInsertRow.ReturnedValue", "{s: O}", "result", OBJ(rowid));
       goto pyexception;
     }
   }
@@ -1101,7 +1097,7 @@ apswvtabUpdate(sqlite3_vtab *pVtab, int argc, sqlite3_value **argv, sqlite3_int6
 pyexception: /* we had an exception in python code */
   assert(PyErr_Occurred());
   sqliteres = MakeSqliteMsgFromPyException(&pVtab->zErrMsg);
-  AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xUpdate", "{s: O, s: i, s: s, s: O}", "self", vtable, "argc", argc, "methodname", methodname, "args", args ? args : Py_None);
+  AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xUpdate", "{s: O, s: i, s: s, s: O}", "self", vtable, "argc", argc, "methodname", methodname, "args", OBJ(args));
 
 finally:
   Py_XDECREF(args);
@@ -1376,7 +1372,7 @@ apswvtabColumn(sqlite3_vtab_cursor *pCursor, sqlite3_context *result, int ncolum
 pyexception: /* we had an exception in python code */
   assert(PyErr_Occurred());
   sqliteres = MakeSqliteMsgFromPyException(&(pCursor->pVtab->zErrMsg)); /* SQLite flaw: errMsg should be on the cursor not the table! */
-  AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xColumn", "{s: O, s: O}", "self", cursor, "res", res ? res : Py_None);
+  AddTraceBackHere(__FILE__, __LINE__, "VirtualTable.xColumn", "{s: O, s: O}", "self", cursor, "res", OBJ(res));
 
 finally:
   Py_XDECREF(res);
@@ -1550,5 +1546,4 @@ method then the first error may mask the second or vice versa.
    receive any constraintargs at all.
 */
 
-#endif /* EXPERIMENTAL */
 /* end of Virtual table code */
